@@ -6,6 +6,7 @@ from django.conf.urls.defaults import patterns, url
 from django.shortcuts import get_object_or_404, render_to_response
 from django.utils.translation import ugettext as _
 from django.utils.translation import get_language
+from django.forms import model_to_dict
 from django.forms.formsets import all_valid
 from django.db import transaction
 from django.http import HttpResponseRedirect
@@ -20,11 +21,34 @@ from reversion.admin import VersionAdmin
 from reversion.revisions import Version, Revision
 import reversion
 
-from easymode.easypublisher.models import EasyPublisherMetaData
+from easymode.easypublisher.models import EasyPublisherMetaData, EasyPublisherModel
 from easymode.tree.admin.relation import ForeignKeyAwareModelAdmin, InvisibleModelAdmin
+from easymode.utils import first_match
+
+__all__ = ('EasyPublisher', 'EasyPublisherInvisibleModelAdmin')
+
+def _eq(self, obj):
+    "Helper to have proper equality for models"
+    field_names = map(lambda x: x.name, self._meta.fields)
+    
+    if not field_names:
+        raise Exception("%s has no fields or hides them very well" % self)
+    
+    for name in field_names:
+        try:
+            if getattr(self, name) != getattr(obj, name):
+                return False
+        except AttributeError:
+            return False
+            
+    return True
 
 class EasyPublisher(VersionAdmin):
-    """docstring for EasyPublisher"""
+    """
+    An admin class that adds approval functionality to the django admin.
+    
+    Drafts will be saved as revisions using reversion.
+    """
     
     object_history_template = "easymode/easypublisher/object_history.html"
     revision_form_template = "easymode/easypublisher/publish_form.html"
@@ -49,8 +73,8 @@ class EasyPublisher(VersionAdmin):
         """Renders the drafts view, listing all drafts"""
         opts = self.model._meta
         action_list = [{"revision": version.revision,
-                        "url": reverse("admin:%s_%s_draft" % (opts.app_label, opts.module_name), args=(version.object_id, version.id))}
-                       for version in self.get_draft_versions(object_id).select_related("revision__user")]
+                        "url": reverse("admin:%s_%s_draft" % (opts.app_label, opts.module_name), args=(version.object_id, version.revision.id))}
+                       for version in self.get_draft_versions(object_id).select_related("revision")]
         context = {
             "action_list": action_list, 
             "title": _("Unpublished items"), 
@@ -87,7 +111,7 @@ class EasyPublisher(VersionAdmin):
         
     @transaction.commit_on_success
     @reversion.revision.create_on_success
-    def publish_view(self, request, object_id, version_id, extra_context=None):
+    def publish_view(self, request, object_id, revision_id, extra_context=None):
         """
         Displays a draft.
         If you have publishing right, the save button will publish the draft.
@@ -95,7 +119,10 @@ class EasyPublisher(VersionAdmin):
         """
         
         obj = get_object_or_404(self.model, pk=object_id)
-        version = get_object_or_404(Version, pk=version_id, object_id=force_unicode(obj.pk))
+        version = get_object_or_404(Version,
+            revision=revision_id,
+            object_id=force_unicode(obj.pk),
+            content_type=ContentType.objects.get_for_model(obj))
         
         if not version.revision.easypublishermetadata_set.filter(language=request.LANGUAGE_CODE):
             request.user.message_set.create(message=_("There is no draft available for language %s") % request.LANGUAGE_CODE)
@@ -107,7 +134,8 @@ class EasyPublisher(VersionAdmin):
             "publish":True,
             'has_draft':True,
             'link_current':True,
-            'extra':0
+            'extra':0,
+            'revision_id': revision_id,
         }
         context.update(extra_context or {})
         return self.render_revision_form(request, obj, version, context, revert=True)
@@ -117,13 +145,19 @@ class EasyPublisher(VersionAdmin):
         Saves the model if you have publishing right, but only makes
         a draft in reversion if you don't
         """
-        
         if request.user.has_perm("easypublisher.can_approve_for_publication"):
             obj.save()
         else:
             reversion.revision.add_meta(EasyPublisherMetaData, status='draft', language=request.LANGUAGE_CODE)
             reversion.revision.comment = "Draft"
-            reversion.revision.post_save_receiver(obj, 0)            
+
+            if not change and hasattr(obj, 'published'):
+                obj.published = False
+                obj.save()
+                obj.published = True
+            
+            reversion.revision.post_save_receiver(obj, 0)
+                
 
     def save_formset(self, request, form, formset, change):
         """
@@ -136,7 +170,16 @@ class EasyPublisher(VersionAdmin):
         else:
             reversion.revision.add_meta(EasyPublisherMetaData, status='draft', language=request.LANGUAGE_CODE)
             instances = formset.save(commit=False)
+            
             for instance in instances:
+                if instance.pk is None:
+                    # reversion puts these models in a set(). If we don't define
+                    # proper equality for them, they will all count as equal, since
+                    # their id's are all the same. This should not have to be done
+                    # for every object, but only once for every type of object.
+                    # however, it really doesn't matter that much for performance
+                    # so this is how it is.
+                    setattr(instance.__class__, '__eq__', _eq)
                 reversion.revision.post_save_receiver(instance, 0)
             
     def has_draft(self, object_id):
@@ -185,19 +228,20 @@ class EasyPublisher(VersionAdmin):
         Retrieves the latest draft that belongs to object_id.
         """
         latest_revision = self.get_latest_draft_revision(object_id)
-        content_type = ContentType.objects.get_for_model(self.model)
-        if latest_revision:
-            latest_drafts = latest_revision.version_set.filter(content_type=content_type).distinct()
-            if len(latest_drafts):
-                return latest_drafts[0]
-            
-        return None
+        return latest_revision
     
     def update_draft(self, version, request):
         """Update the status of the draft belonging to this version"""
         for metadata in version.revision.easypublishermetadata_set.all():
             if request.user.has_perm("easypublisher.can_approve_for_publication"):                
                 metadata.status = 'published'
+                # save all other drafts for this object as declined, because we
+                # chose to save a different one
+                for other in EasyPublisherMetaData.objects.filter(
+                    revision__version__object_id=version.object_id, 
+                    revision__version__content_type=version.content_type):
+                    other.status = 'declined'
+                    other.save()
             else:
                 metadata.status = 'updated'
             metadata.save()
@@ -238,14 +282,36 @@ class EasyPublisher(VersionAdmin):
                 prefixes[prefix] = prefixes.get(prefix, 0) + 1
                 if prefixes[prefix] != 1:
                     prefix = "%s-%s" % (prefix, prefixes[prefix])
+                
                 formset = FormSet(request.POST, request.FILES,
                                   instance=new_object, prefix=prefix,
                                   queryset=inline.queryset(request))
+
+                # _ _ _ ____ ___ ____ _  _    ____ _  _ ___ 
+                # | | | |__|  |  |    |__|    |  | |  |  |  
+                # |_|_| |  |  |  |___ |  |    |__| |__|  |
+                # this is not copy pasted:
                 # Strip extra empty forms from the formset.
-                num_forms = formset.total_form_count() - formset.extra
-                del formset.forms[num_forms:]
+                empty_forms = []
+                post_keys = request.POST.keys()
+                for f in formset.forms:
+                    # the forms that can be removed, are not in the request.POST
+                    # we can find them by their prefix
+                    matches_prefix = lambda var: var.find(f.prefix) == 0 or None
+                    if first_match(matches_prefix, post_keys) is None:
+                        empty_forms.append(f)
+                    else:
+                        # the form must be cleaned, which they aren't yet because
+                        # we just created them
+                        f.full_clean()
+                
+                # modify form settings of formset.
+                num_forms = formset.total_form_count() - len(empty_forms)
+                formset.forms = filter(lambda x: x not in empty_forms, formset.forms)
                 formset.total_form_count = lambda: num_forms
-                # Add this hacked formset to the form.
+                # end of non copy pasted piece
+                
+                # Add this hacked formset to the form.                
                 formsets.append(formset)
             if all_valid(formsets) and form_validated:
                 self.save_model(request, new_object, form, change=True)
@@ -294,22 +360,48 @@ class EasyPublisher(VersionAdmin):
                 except AttributeError:
                     # This is a GenericInlineFormset, or similar.
                     fk_name = FormSet.ct_fk_field.name
-                related_versions = dict([(related_version.object_id, related_version)
+                
+                # _ _ _ ____ ___ ____ _  _    ____ _  _ ___ 
+                # | | | |__|  |  |    |__|    |  | |  |  |  
+                # |_|_| |  |  |  |___ |  |    |__| |__|  |
+                # this is not copy pasted:
+                related_versions = [(related_version.object_id, related_version)
                                          for related_version in revision_versions
                                          if ContentType.objects.get_for_id(related_version.content_type_id).model_class() == FormSet.model
-                                         and unicode(related_version.field_dict[fk_name]) == unicode(object_id)])
+                                         and unicode(related_version.field_dict[fk_name]) == unicode(object_id)]
+                
+                # all items that do not have their id filled in must stay.
+                # if they do in fact have an id, we only want the last one,
+                # no duplicates.
+                related_versions = [(key, value) for (key, value) in related_versions if key == 'None'] + \
+                    dict([(key, value) for (key, value) in related_versions if key != 'None']).items()
+                
                 initial = []
                 for related_obj in formset.queryset:
-                    if unicode(related_obj.pk) in related_versions:
-                        initial.append(related_versions.pop(unicode(related_obj.pk)).field_dict)
+                    related_versions_dict = dict(related_versions)
+                    if unicode(related_obj.pk) in related_versions_dict:
+                        initial.append(related_versions_dict.pop(unicode(related_obj.pk)).field_dict)
                     else:
                         initial_data = model_to_dict(related_obj)
                         initial_data["DELETE"] = True
                         initial.append(initial_data)
-                for related_version in related_versions.values():
+                for (key, related_version) in related_versions:
                     initial_row = related_version.field_dict
-                    del initial_row["id"]
-                    initial.append(initial_row)
+                    try:
+                        del initial_row["id"]
+                    except KeyError:
+                        # when multiple inlines with the same model but a different
+                        # querysets are on a page, it could happen that we allready
+                        # deleted the id.
+                        pass
+                    
+                    # we didn't convert related_versions to a dict so there can be
+                    # duplicate keys in the thingy
+                    if not initial_row in initial:
+                        initial.append(initial_row)
+                
+                # end of non copy pasted piece
+                
                 # Reconstruct the forms with the new revision data.
                 formset.initial = initial
                 formset.forms = [formset._construct_form(n) for n in xrange(len(initial))]
@@ -344,7 +436,7 @@ class EasyPublisher(VersionAdmin):
                         "has_change_permission": self.has_change_permission(request, obj),
                         "has_delete_permission": self.has_delete_permission(request, obj),
                         "has_file_field": True,
-                        "has_absolute_url": False,
+                        "has_absolute_url": getattr(self.model, 'get_absolute_url', False),
                         "ordered_objects": opts.get_ordered_objects(),
                         "form_url": mark_safe(request.path),
                         "opts": opts,
@@ -355,6 +447,7 @@ class EasyPublisher(VersionAdmin):
                         "change_url": reverse("admin:%s_%s_change" % (opts.app_label, opts.module_name), args=(obj.pk,)),
                         "history_url": reverse("admin:%s_%s_history" % (opts.app_label, opts.module_name), args=(obj.pk,)),
                         "recoverlist_url": reverse("admin:%s_%s_recoverlist" % (opts.app_label, opts.module_name))})
+        
         # Render the form.
         if revert:
             form_template = self.revision_form_template
@@ -421,12 +514,16 @@ def _add_invisible_model_admin_behaviour(method):
         if extra_context:
             defaults.update(extra_context)
     
-        return method(self, request, object_id, extra_context)
+        return method(self, request, object_id, defaults)
     
     return altered_view
 
 class EasyPublisherInvisibleModelAdmin(EasyPublisher, InvisibleModelAdmin):
-    """fixes the collision between EasyPublisher and InvisibleModelAdmin"""
+    """
+    fixes the collision between EasyPublisher and InvisibleModelAdmin
+    
+    see :mod:`easymode.tree.admin.relation`
+    """
         
     def get_model_perms(self, request):
         perms = super(EasyPublisherInvisibleModelAdmin, self).get_model_perms(request)
