@@ -1,21 +1,28 @@
-from django.db import models
-from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes import generic
+from django.contrib.contenttypes.models import ContentType
+from django.db import models
 
-from easymode.tree.decorators import toxml
-from easymode.admin.models.fields import *
+from easymode.admin.models.fields import DiocoreTextField, DiocoreHTMLField, FlashUrlField, DiocoreCharField
 from easymode.i18n.decorators import I18n
+from easymode.tree.decorators import toxml
+from easymode.easypublisher.models import EasyPublisherModel
+
 
-@I18n('charfield')
 @toxml
+@I18n('charfield')
 class TestModel(models.Model):
     """test model"""
 
-    charfield = models.CharField(max_length=255)
+    charfield = models.CharField(max_length=255, unique=True)
+    tags = models.ManyToManyField('TagModel', related_name='testmodels')
     
     def __unicode__(self):
         return u"%s%s - %s" % (self.__class__.__name__, self.pk, self.charfield)
 
+    def natural_key(self):
+        return (self.charfield,)
+    
+
 class TestSubModel(models.Model):
     """hangs onto testmodel"""
 
@@ -25,7 +32,8 @@ class TestSubModel(models.Model):
 
     def __unicode__(self):
         return u"%s%s - %s" % (self.__class__.__name__, self.pk, self.subcharfield)
-    
+
+
 class TestSecondSubmodel(models.Model):
     """hangs on to TestModel as well"""
     
@@ -35,6 +43,7 @@ class TestSecondSubmodel(models.Model):
     def __unicode__(self):
         return self.ultrafield
 
+
 class TestSubSubModel(models.Model):
     """Even subier sub model"""
     
@@ -44,21 +53,12 @@ class TestSubSubModel(models.Model):
     def __unicode__(self):
         return self.subsubcharfield
 
-@toxml
-class UrlFieldTestModel(models.Model):
-    title    = models.CharField(max_length=256)
-    revision = models.PositiveIntegerField()
-    url      = FlashUrlField(max_length=256)
-# 
-# @toxml
-# class IncludeFileFieldTestModel(models.Model):
-#     title = models.CharField(max_length=256)
-#     file  = IncludeFileField(base=os.path.dirname(__file__), relative_path='hax')
-# 
-# @toxml
-# class RemoteIncludeFieldModel(models.Model):
-#     title = models.CharField(max_length=256)
-#     url   = RemoteIncludeField(interval=60)
+
+@I18n('value')
+class TagModel(models.Model):
+    
+    value = models.CharField(max_length=23)
+
 
 @toxml
 @I18n('title', 'description')
@@ -72,6 +72,7 @@ class TestL10nModel(models.Model):
     def __unicode__(self):
         return u"%s%s" % (self.__class__.__name__, self.pk)
 
+
 class GenericRelatedModel(models.Model):
     """it is used in a generic relation"""
 
@@ -84,6 +85,7 @@ class GenericRelatedModel(models.Model):
     def __unicode__(self):
         return u"%s%s" % (self.__class__.__name__, self.pk)
 
+
 @toxml
 class TestGenericFkModel(models.Model):
     """it is used for testing"""
@@ -94,4 +96,15 @@ class TestGenericFkModel(models.Model):
     def __unicode__(self):
         return u"%s%s" % (self.__class__.__name__, self.pk)
 
-
+class TestEasypublisherModel(EasyPublisherModel):
+    label = models.CharField(max_length=200)
+    nogeenlabel = models.CharField(max_length=200)
+    def __unicode__(self):
+        return "%s" % self.label
+    
+class TestEasypublisherRelatedModel(models.Model):
+    parent = models.ForeignKey(TestEasypublisherModel)
+    
+    extra = models.CharField(max_length=200)
+    def __unicode__(self):
+        return "%s" % self.extra
\ No newline at end of file
