@@ -7,9 +7,6 @@ ugettext = lambda s: s
 DEBUG = True
 TEMPLATE_DEBUG = DEBUG
 
-# some tests fail because they are not yet implemented!
-SKIPPED_TESTS = ('test_generic_relations_also_work', 'test_tests_are_implemented')
-
 PROJECT_DIR = os.path.dirname(__file__)
 
 ADMINS = (
@@ -24,6 +21,8 @@ DATABASE_USER = ''             # Not used with sqlite3.
 DATABASE_PASSWORD = ''         # Not used with sqlite3.
 DATABASE_HOST = ''             # Set to empty string for localhost. Not used with sqlite3.
 DATABASE_PORT = ''             # Set to empty string for default. Not used with sqlite3.
+# TEST_DATABASE_NAME = 'example_test_data.sqlite'
+# DATABASE_NAME = TEST_DATABASE_NAME
 
 # Local time zone for this installation. Choices can be found here:
 # http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
